from typing import List

from anndata import AnnData
from scipy.sparse import spmatrix


def build_prediction_column_name(annotation_prefix, suffix=None) -> str:
    return "_".join(filter(None, [annotation_prefix, 'predicted', suffix]))


def record_predictions(query_dataset, annotation_prefix, pred_labels):
    query_dataset.obs[build_prediction_column_name(annotation_prefix)] = pred_labels


def record_prediction_run_metadata(query_dataset: AnnData, annotation_prefix: str, model_url: str):
    query_dataset.uns[f"{annotation_prefix}_predictions"] = {'model_url': model_url}


def extract_raw_counts_adata(ad, raw_counts_layer_name) -> spmatrix:
    if raw_counts_layer_name:
        return ad.layers[raw_counts_layer_name]
    elif ad.raw:
        return ad.raw.X
    else:
        return ad.X


def extract_gene_names(ad, gene_column_name, organism) -> List[str]:
    # TODO: Do we need to use ad.raw.var if extract_raw_counts_adata() finds raw counts in ad.raw.X?
    var_names = ad.var[gene_column_name] if gene_column_name else ad.var_names

    if organism == 'Mus musculus':
        var_names_upcased = [n.upper() for n in var_names]
        print('Please note you are mapping to a human reference atlas.'
             ' Variable names are cast to uppercase to enable this.') #-> replace with better logging?
        return var_names_upcased
    else:
        return var_names