import json

import scanpy
from mlflow.pyfunc import PyFuncOutput, PyFuncInput
from pandas import DataFrame

from cell_type.scvi_models.predict import annotate


class CellTypeAnnotationModelWrapper:

    def __init__(self, artifacts_path):
        print(f"artifacts_path={artifacts_path}")
        self.model_artifacts_path = artifacts_path

    def predict(self, input: PyFuncInput) -> PyFuncOutput:
        if isinstance(input, DataFrame):
            input = input.iloc[0][0]
        input = json.loads(input)
        print(f"input={input}")
        input_h5ad_path = input.get('query_dataset_h5ad_path')
        query_dataset = scanpy.read_h5ad(input_h5ad_path)
        # TODO(atolopko): For cellxgene schema-compliant H5ADs, we should be able to obtain organism arg directly.
        del input['query_dataset_h5ad_path']
        output_h5ad_path = input.get('output_h5ad_path')
        if 'output_h5ad_path' in input:
            del input['output_h5ad_path']

        annotate(query_dataset,
                 output_h5ad_path,
                 self.model_artifacts_path,
                 **input)
        return [output_h5ad_path or input_h5ad_path]
